<?php
/**
 * 购物车
 * Created by PhpStorm.
 */
namespace JinDouYun\Controller\Cart;

use Mall\Framework\Core\ErrorCode;
use JinDouYun\Controller\BaseController;

use JinDouYun\Model\Cart\MCart;
use Mall\Framework\Core\StatusCode;

class ApiCart extends BaseController
{
    private $objMCart;

    /**
     * ApiCart constructor.
     * @param bool $isCheckAcl
     * @param bool $isMustLogin
     * @param bool $checkToken
     * @param bool $getAreaCode
     * @throws \Exception
     */
    public function __construct($isCheckAcl = false, $isMustLogin = true ,$checkToken=true,$getAreaCode=true)
    {
        parent::__construct($isCheckAcl, $isMustLogin, $checkToken = true, $getAreaCode = true);
        $this->objMCart = new MCart($this->onlineUserId, $this->onlineEnterpriseId,true,$this->areaCode);
    }

    /**
     * 添加购物车,公共字段
     *
     * @return array
     */
    public function commonFieldFilter()
    {
        $params = $this->request->getRawJson();
        if (empty($params)) {
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }

        $cartData = [
            'goodsData' => isset($params['goodsData']) ? $params['goodsData'] : [], //商品数据
        ];

        foreach ($cartData as $key => $value) {
            if (empty($value) && $value !== 0) {
                parent::sendOutput($key . '参数错误', ErrorCode::$paramError);
            }
        }

        $goodsData = [];
        foreach ($cartData['goodsData'] as $key => $val) {
            $goodsData[$key] = [
                'goodsBasicId' => isset($val['goodsBasicId']) ? $val['goodsBasicId'] : '',
                'goodsId'      => isset($val['goodsId']) ? $val['goodsId'] : '',
                'skuId'        => isset($val['skuId']) ? $val['skuId'] : '',
                'buyNum'       => isset($val['buyNum']) ? $val['buyNum'] : '',
                'shopId'       => isset($val['shopId']) ? $val['shopId'] : '',
                'source'       => isset($val['source']) ? $val['source'] : '',
            ];

            foreach ($goodsData[$key] as $k => $v) {
                if (empty($v)) {
                    parent::sendOutput($k . '参数错误', ErrorCode::$paramError);
                }
            }
            $goodsData[$key]['goodsCode'] = createCode(StatusCode::$code['goodsBasic']['prefix'], $goodsData[$key]['goodsId'], StatusCode::$code['goodsBasic']['length']);
        }
        $cartData['goodsData'] = $goodsData;//过滤后数据
        return $cartData;
    }

    /**
     * 加入购物车
     * @throws \Exception
     */
    public function addCart()
    {
        $cartData = $this->commonFieldFilter();
        $result = $this->objMCart->addCartApi($cartData);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 更新商品数量(购物车内操作)
     * @throws \Exception
     */
    public function updateBuyNum()
    {
        $params['cartId'] = $this->request->param('request_id');
        $params['buyNum'] = $this->request->param('buyNum');

        foreach ($params as $key => $value) {
            if (empty($value) && $value !== 0) {
                $this->sendOutput($key . '参数错误', ErrorCode::$paramError);
            }
        }
        $result = $this->objMCart->updateBuyNumApi($params);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 获取用户购物车数据
     * @throws \Exception
     */
    public function getCartByUserCenterId()
    {
        $result = $this->objMCart->getCartByUserCenterIdApi();
        if ($result->isSuccess()) {
            $returnData = $result->getData();
            $pageData = [
                'pageIndex' => 0,
                'pageSize'  => 0,
                'pageTotal' => $returnData['total']
            ];
            parent::sendOutput($returnData['data'], 0, $pageData);
        }
        parent::sendOutput($result->getData(), ErrorCode::$dberror);
    }

    /**
     * 更新选中状态
     * @throws \Exception
     */
    public function updateSelection()
    {
        $paramsData = $this->request->getRawJson();
        $params = [
            'selection' => isset($paramsData['selection']) ? $paramsData['selection'] : '',//4=>未选中 5=>已选中
            'type'      => isset($paramsData['type']) ? $paramsData['type'] : 0,//type 1单个 2店铺 3全选
        ];

        foreach ($params as $key => $value) {
            if (empty($value)) {
                $this->sendOutput($key . '参数错误', ErrorCode::$paramError);
            }
        }
        $params['id'] = isset($paramsData['cartId']) ? $paramsData['cartId'] : 0;//cartId
        $params['shopId'] = isset($paramsData['shopId']) ? $paramsData['shopId'] : 0;//店铺id

        $result = $this->objMCart->updateSelection($params);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 删除购物车中商品(可批量)
     */
    public function delCart()
    {
        $params = $this->request->getRawJson();
        $paramsData = [
            'id' => isset($params['cartId']) ? $params['cartId'] : 0,
        ];

        foreach ($paramsData as $key => $value) {
            if (empty($value) && $value !== 0) {
                $this->sendOutput($key . '参数错误', ErrorCode::$paramError);
            }
        }
        $result = $this->objMCart->delCart($paramsData);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 确认订单接口
     * @throws \Exception
     */
    public function confirmationOfOrder()
    {
        $vipCardId = $this->request->param('vipCardId');//会员卡id
        $userCouponId=$this->request->param('userCouponId');//优惠券id
        $addressId = $this->request->param('addressId');//收货地址id
        $deliveryId = $this->request->param('deliveryId');//配送方式id
        $proporties = $this->request->param('proporties');//店铺对应对应的收货地址以及配送方式
        $result = $this->objMCart->confirmationOfOrder($userCouponId,$vipCardId,$addressId,$deliveryId);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 商品列表用
     * @throws \Exception
     */
    public function updateCartNum()
    {
        $paramsData = $this->request->getRawJson();
        if (empty($paramsData)) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }
        $params = [
            'goodsId' => $this->request->param('request_id'),
            'buyNum'  => isset($paramsData['buyNum']) ? $paramsData['buyNum'] : '',
            'skuId'   => isset($paramsData['skuId']) ? $paramsData['skuId'] : '',
        ];
        foreach ($params as $key => $value) {
            if (empty($value) && $value !== 0) {
                $this->sendOutput($key . '参数错误', ErrorCode::$paramError);
            }
        }
        $result = $this->objMCart->updateCartNum($params);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 获取当前用户会员卡
     */
    public function getVipCard()
    {
        $result = $this->objMCart->getVipCard([]);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * Doc: (des="立即购买")
     * User: XMing
     * Date: 2021/1/22
     * Time: 4:01 下午
     * @throws \Exception
     */
    public function buyNow()
    {
        $params = $this->request->getRawJson();
        if (empty($params)) {
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }
        if (!isset($params['lists']) || empty($params['lists'])){
            parent::sendOutput('请选择商品',ErrorCode::$paramError);
        }
        $lists = getArrayItem($params,'lists',[]);


        $data = [
            'sourceType' => getArrayItem($params,'sourceType',0),
            'comBinId'   => getArrayItem($params,'couponId',0),
            'couponId'   => getArrayItem($params,'couponId',0),
            'addressId'  => getArrayItem($params,'addressId',0),
            'deliveryId' => getArrayItem($params,'deliveryId',0),
        ];
        $result = $this->objMCart->buyNow($lists,$data);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 普通商品立即购买
     */
    public function goodsBuyNow()
    {
        $params = $this->request->getRawJson();
        if (empty($params)) {
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }
        $goods = [
            'goodsId' => isset($params['goodsId']) ? $params['goodsId'] : '',
            'goodsBasicId' => getArrayItem($params, 'goodsBasicId', 0),
            'skuId'   => isset($params['skuId']) ? $params['skuId'] : '',
            'buyNum'  => isset($params['buyNum']) ? $params['buyNum'] : '',
            'shopId'  => getArrayItem($params, 'shopId', 0),
            'selection' => StatusCode::$standard,
        ];
        foreach ($goods as $key => $val) {
            if (empty($val)) {
                parent::sendOutput($key . '参数错误', ErrorCode::$paramError);
            }
        }
        $goods['activityId'] = getArrayItem($params, 'activityId', 0);

        $data['vipCardId'] = getArrayItem($params, 'vipCardId', 0);
        $data['couponId'] = isset($params['couponId']) ? $params['couponId'] : 0;
        $data['addressId'] = isset($params['addressId']) ? $params['addressId'] : 0;
        $data['deliveryId'] = isset($params['deliveryId']) ? $params['deliveryId'] : 0;

        $result = $this->objMCart->goodsBuyNow($goods,$data);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

}