<?php
/**
 * 配送Controller
 * Created by PhpStorm.
 * User: haoren
 * Date: 2021/04/20
 * Time: 16:00
 */

namespace JinDouYun\Controller\Delivery;

use Mall\Framework\Core\ErrorCode;
use Mall\Framework\Core\StatusCode;

use JinDouYun\Controller\BaseController;

use JinDouYun\Model\Delivery\MDelivery;

class Delivery extends BaseController
{
    private $objMDelivery;

    public function __construct($isCheckAcl = true, $isMustLogin = true)
    {
        parent::__construct($isCheckAcl, $isMustLogin);
        $this->objMDelivery = new MDelivery($this->onlineEnterpriseId, $this->onlineUserId);
    }

    /**
     * 获取参数
     * @return array
     */
    public function commonFieldFilterDeliveryRoute()
    {
        $params = $this->request->getRawJson();
        if (empty($params)) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }

        $data = [
            "name" => isset($params['name']) ? $params['name'] : '',
        ];

        foreach ($data as $key => $value) {
            if (empty($value) && $value !== 0) {
                $this->sendOutput($key . '参数错误', ErrorCode::$paramError);
            }
        }
        isset($params['warehouseId']) && $data['warehouseId'] = $params['warehouseId'];
        isset($params['shopId']) && $data['shopId'] = $params['shopId'];
        isset($params['driverId']) && $data['driverId'] = trim($params['driverId'],',');
        isset($params['customerId']) && $data['customerId'] = $params['customerId'];
        isset($params['desc']) && $data['desc'] = $params['desc'];

        return $data;
    }

    /**
     * 配送线路添加
     */
    public function addDeliveryRoute()
    {
        $data = $this->commonFieldFilterDeliveryRoute();
        $result = $this->objMDelivery->addDeliveryRoute($data);
        if (!$result->isSuccess()) {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
        parent::sendOutput($result->getData());
    }

    /**
     * 配送线路删除
     */
    public function deleteDeliveryRoute()
    {
        $id = $this->request->param('request_id');
        if (empty($id)) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }

        $update = [
            'deleteStatus' => StatusCode::$delete
        ];
        $result = $this->objMDelivery->updateDeliveryRoute($update, ['id' => $id]);
        if (!$result->isSuccess()) {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
        parent::sendOutput($result->getData());
    }

    /**
     * 配送线路修改
     */
    public function updateDeliveryRoute()
    {
        $id = $this->request->param('request_id');
        if (empty($id)) {
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }
        $update = $this->commonFieldFilterDeliveryRoute();
        $result = $this->objMDelivery->updateDeliveryRoute($update, ['id' => $id]);
        if (!$result->isSuccess()) {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
        parent::sendOutput($result->getData());
    }

    /**
     * 配送线路启用/禁用
     */
    public function enableDeliveryRoute()
    {
        $id = $this->request->param('request_id');
        if (empty($id)) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }
        $params = $this->request->getRawJson();
        $data = [
            'enableStatus' => isset($params['enableStatus']) ? $params['enableStatus'] : '',
        ];
        foreach ($data as $key => $value) {
            if (empty($value) && $value !== 0) {
                $this->sendOutput($key . '参数错误', ErrorCode::$paramError);
            }
        }
        $result = $this->objMDelivery->updateDeliveryRoute($data, ['id' => $id]);
        if (!$result->isSuccess()) {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
        parent::sendOutput($result->getData());
    }

    /**
     * 配送线路列表
     */
    public function getAllDeliveryRoute()
    {
        $params = $this->request->getRawJson();

        $pageParams = pageToOffset(isset($params['page']) ? $params['page'] : 1, isset($params['pageSize']) ? $params['pageSize'] : 10);
        $selectParams['limit'] = $pageParams['limit'];
        $selectParams['offset'] = $pageParams['offset'];

        if(isset($params['start']) && !empty($params['start']) && isset($params['end']) && !empty($params['end'])){
            $selectParams['createTime']['start'] = $params['start'];
            $selectParams['createTime']['end'] = $params['end'];
        }

        $result = $this->objMDelivery->getAllDeliveryRoute($selectParams);
        if (!$result->isSuccess()) {
            parent::sendOutput($result->getData(), ErrorCode::$dberror);
        }
        $returnData = $result->getData();
        $pageData = [
            'pageIndex' => $params['page'],
            'pageSize' => $params['pageSize'],
            'pageTotal' => $returnData['total'],
        ];
        parent::sendOutput($returnData['data'], 0, $pageData);
    }

    /**
     * 配送线路详情
     */
    public function getDeliveryRouteInfo()
    {
        $params['id'] = $this->request->param('request_id');
        if (empty($params['id'])) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }

        $result = $this->objMDelivery->getDeliveryRouteInfo($params);
        if (!$result->isSuccess()) {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
        parent::sendOutput($result->getData());
    }

    /**
     * 配送线路客户
     */
    public function getDeliveryRouteCustomer()
    {
        $params = $this->request->getRawJson();

        $pageParams = pageToOffset(isset($params['page']) ? $params['page'] : 1, isset($params['pageSize']) ? $params['pageSize'] : 10);
        $selectParams['limit'] = $pageParams['limit'];
        $selectParams['offset'] = $pageParams['offset'];
        if(!isset($params['id']) || empty($params['id'])){
            parent::sendOutput('id参数错误', ErrorCode::$paramError);
        }
        $selectParams['routeId'] = $params['id'];

        $result = $this->objMDelivery->getDeliveryRouteCustomer($selectParams);
        if (!$result->isSuccess()) {
            parent::sendOutput($result->getData(), ErrorCode::$dberror);
        }
        $returnData = $result->getData();
        $pageData = [
            'pageIndex' => $params['page'],
            'pageSize' => $params['pageSize'],
            'pageTotal' => $returnData['total'],
        ];
        parent::sendOutput($returnData['data'], 0, $pageData);
    }


    /**
     * 配送线路客户排序
     */
    public function updateDeliveryRouteCustomer()
    {
        $id = $this->request->param('request_id');
        if (empty($id)) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }
        $params = $this->request->getRawJson();
        $data = [
            'sort' => isset($params['sort']) ? $params['sort'] : '',
        ];
        foreach ($data as $key => $value) {
            if (empty($value) && $value !== 0) {
                $this->sendOutput($key . '参数错误', ErrorCode::$paramError);
            }
        }
        $result = $this->objMDelivery->updateDeliveryRouteCustomer($data, ['id' => $id]);
        if (!$result->isSuccess()) {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
        parent::sendOutput($result->getData());
    }

}