<?php
/**
 * 积分商品Controller
 * Created by PhpStorm.
 * User: haoren
 * Date: 2021/03/25
 * Time: 16:00
 */

namespace JinDouYun\Controller\Integral;

use Mall\Framework\Core\ErrorCode;
use Mall\Framework\Core\StatusCode;

use JinDouYun\Controller\BaseController;

use JinDouYun\Model\Integral\MIntegralGoods;

class IntegralGoods extends BaseController
{
    private $objMIntegralGoods;

    public function __construct($isCheckAcl = true, $isMustLogin = true)
    {
        parent::__construct($isCheckAcl, $isMustLogin);
        $this->objMIntegralGoods = new MIntegralGoods($this->onlineEnterpriseId, $this->onlineUserId);
    }

    /**
     * 获取参数
     * @return array
     */
    public function commonFieldFilter()
    {
        $params = $this->request->getRawJson();
        if (empty($params)) {
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }
        $data = [
            'name' => isset($params['name']) ? $params['name'] : '',
            'images' => isset($params['images']) ? json_encode($params['images']) : [],
            'integral' => isset($params['integral']) ? $params['integral'] : '',
            'category' => isset($params['category']) ? $params['category'] : '',
        ];
        foreach($data as $key => $value){
            if(empty($value)){
                parent::sendOutput($key.'参数错误', ErrorCode::$paramError);
            }
        }
        !empty($this->shopId) && $data['shopId'] = $this->shopId;
        isset($params['sort']) && $data['sort'] = $params['sort'];
        isset($params['limit']) && $data['limit'] = $params['limit'];
        isset($params['num']) && $data['num'] = $params['num'];
        isset($params['desc']) && $data['desc'] = $params['desc'];
        isset($params['details']) && $data['details'] = $params['details'];
        return $data;
    }

    /**
     * 积分商品添加
     */
    public function addIntegralGoods()
    {
        $data = $this->commonFieldFilter();
        $result = $this->objMIntegralGoods->addIntegralGoods($data);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        } else {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
    }

    /**
     * 积分商品删除
     */
    public function deleteIntegralGoods()
    {
        $id = $this->request->param('request_id');
        if (empty($id)) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }
        $result = $this->objMIntegralGoods->deleteIntegralGoods(['id' => $id]);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        } else {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
    }

    /**
     * 积分商品禁用
     */
    public function enableIntegralGoods()
    {
        $params['id'] = $this->request->param('request_id');
        if (empty($params)) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }
        $result = $this->objMIntegralGoods->enableIntegralGoods($params);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        } else {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
    }

    /**
     * 积分商品修改
     */
    public function updateIntegralGoods()
    {
        $updateData = self::commonFieldFilter();
        $id = $this->request->param('request_id');
        if(empty($id)){
            parent::sendOutput('id参数为空', ErrorCode::$paramError);
        }
        $result = $this->objMIntegralGoods->updateIntegralGoods($updateData, ['id' => $id]);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        } else {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
    }

    /**
     * 积分商品列表
     */
    public function getAllIntegralGoods()
    {
        $params = $this->request->getRawJson();
        if (empty($params)) {
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }

        $pageParams = pageToOffset($params['page'] ?: 1, $params['pageSize'] ?: 10);
        $selectParams['limit'] = $pageParams['limit'];
        $selectParams['offset'] = $pageParams['offset'];

        if(isset($params['search']) && !empty($params['search'])){
            $selectParams['search'] = $params['search'];
        }

        if(!empty($this->shopId)){
            $selectParams['shopId'] = $this->shopId;
        }

        $result = $this->objMIntegralGoods->getAllIntegralGoods($selectParams);
        if ($result->isSuccess()) {
            $returnData = $result->getData();
            $pageData = [
                'pageIndex' => $params['page'],
                'pageSize' => $params['pageSize'],
                'pageTotal' => $returnData['total'],
            ];
            parent::sendOutput($returnData['data'], 0, $pageData);
        } else {
            parent::sendOutput($result->getData(), ErrorCode::$dberror);
        }
    }

    /**
     * 积分商品详情
     */
    public function getIntegralGoodsInfo()
    {
        $params['id'] = $this->request->param('request_id');
        if (empty($params['id'])) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }
        $result = $this->objMIntegralGoods->getIntegralGoodsInfo($params);

        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        } else {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
    }


    /**
     * 积分商品兑换修改
     */
    public function updateIntegralGoodsExchange()
    {
        $id = $this->request->param('request_id');
        if(empty($id)){
            parent::sendOutput('id参数为空', ErrorCode::$paramError);
        }
        $params = $this->request->getRawJson();
        if (empty($params)) {
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }
        $data = [
            'status' => isset($params['status']) ? $params['status'] : '',
        ];
        foreach($data as $key => $value){
            if(empty($value)){
                parent::sendOutput($key.'参数错误', ErrorCode::$paramError);
            }
        }
        isset($params['reason']) && $data['reason'] = $params['reason'];

        $result = $this->objMIntegralGoods->updateIntegralGoodsExchange($data, ['id' => $id]);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        } else {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
    }

    /**
     * 积分商品兑换列表
     */
    public function getAllIntegralGoodsExchange()
    {
        $params = $this->request->getRawJson();
        if (empty($params)) {
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }

        $pageParams = pageToOffset($params['page'] ?: 1, $params['pageSize'] ?: 10);
        $selectParams['limit'] = $pageParams['limit'];
        $selectParams['offset'] = $pageParams['offset'];

        if(isset($params['search']) && !empty($params['search'])){
            $selectParams['search'] = $params['search'];
        }

        if(isset($params['status']) && !empty($params['status'])){
            $selectParams['status'] = $params['status'];
        }

        if(!empty($this->shopId)){
            $selectParams['shopId'] = $this->shopId;
        }

        $result = $this->objMIntegralGoods->getAllIntegralGoodsExchange($selectParams);
        if ($result->isSuccess()) {
            $returnData = $result->getData();
            $pageData = [
                'pageIndex' => $params['page'],
                'pageSize' => $params['pageSize'],
                'pageTotal' => $returnData['total'],
            ];
            parent::sendOutput($returnData['data'], 0, $pageData);
        } else {
            parent::sendOutput($result->getData(), ErrorCode::$dberror);
        }
    }

    /**
     * 积分商品兑换详情
     */
    public function getIntegralGoodsExchangeInfo()
    {
        $params['id'] = $this->request->param('request_id');
        if (empty($params['id'])) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }
        $result = $this->objMIntegralGoods->getIntegralGoodsExchangeInfo($params);

        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        } else {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
    }

    /**
     * 积分规则添加
     */
    public function addIntegralRule()
    {
        $params = $this->request->getRawJson();
        if (empty($params)) {
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }
        $data = [];

        isset($params['title']) && $data['title'] = $params['title'];
        isset($params['amount']) && $data['amount'] = $params['amount'];
        isset($params['integral']) && $data['integral'] = $params['integral'];
        isset($params['goods']) && $data['goods'] = json_encode($params['goods']);
        isset($params['startMoney']) && $data['startMoney'] = json_encode($params['startMoney']);

        $result = $this->objMIntegralGoods->addIntegralRule($data);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        } else {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
    }

    /**
     * 积分规则删除
     */
    public function deleteIntegralRule()
    {
        $id = $this->request->param('request_id');
        if (empty($id)) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }
        $result = $this->objMIntegralGoods->deleteIntegralRule(['id' => $id]);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        } else {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
    }

    /**
     * 积分规则禁用
     */
    public function enableIntegralRule()
    {
        $params['id'] = $this->request->param('request_id');
        if (empty($params)) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }
        $result = $this->objMIntegralGoods->enableIntegralRule($params);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        } else {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
    }

    /**
     * 积分规则修改
     */
    public function updateIntegralRule()
    {
        $id = $this->request->param('request_id');
        if(empty($id)){
            parent::sendOutput('id参数为空', ErrorCode::$paramError);
        }

        $params = $this->request->getRawJson();
        if (empty($params)) {
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }
        $data = [];

        isset($params['title']) && $data['title'] = $params['title'];
        isset($params['amount']) && $data['amount'] = $params['amount'];
        isset($params['integral']) && $data['integral'] = $params['integral'];
        isset($params['goods']) && $data['goods'] = json_encode($params['goods']);
        isset($params['startMoney']) && $data['startMoney'] = json_encode($params['startMoney']);

        $result = $this->objMIntegralGoods->updateIntegralRule($data, ['id' => $id]);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        } else {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
    }

    /**
     * 积分规则列表
     */
    public function getAllIntegralRule()
    {
        $params = $this->request->getRawJson();
        if (empty($params)) {
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }

        $pageParams = pageToOffset($params['page'] ?: 1, $params['pageSize'] ?: 10);
        $selectParams['limit'] = $pageParams['limit'];
        $selectParams['offset'] = $pageParams['offset'];

        if(isset($params['search']) && !empty($params['search'])){
            $selectParams['search'] = $params['search'];
        }

        if(isset($params['enableStatus']) && !empty($params['enableStatus'])){
            $selectParams['enableStatus'] = $params['enableStatus'];
        }

        if(isset($params['star']) && !empty($params['star']) && isset($params['end']) && !empty($params['end'])){
            $selectParams['createTime'] = [
                'star' => $params['star'],
                'end' => $params['end']
            ];
        }

        if(!empty($this->shopId)){
            $selectParams['shopId'] = $this->shopId;
        }

        $result = $this->objMIntegralGoods->getAllIntegralRule($selectParams);
        if ($result->isSuccess()) {
            $returnData = $result->getData();
            $pageData = [
                'pageIndex' => $params['page'],
                'pageSize' => $params['pageSize'],
                'pageTotal' => $returnData['total'],
            ];
            parent::sendOutput($returnData['data'], 0, $pageData);
        } else {
            parent::sendOutput($result->getData(), ErrorCode::$dberror);
        }
    }

    /**
     * 积分规则详情
     */
    public function getIntegralRuleInfo()
    {
        $params['id'] = $this->request->param('request_id');
        if (empty($params['id'])) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }
        $result = $this->objMIntegralGoods->getIntegralRuleInfo($params);

        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        } else {
            parent::sendOutput($result->getData(), $result->getErrorCode());
        }
    }

}