<?php
/**
 * 优惠券
 * Created by PhpStorm.
 * User: XiaoMing
 * Date: 2019/12/13
 * Time: 16:25
 */

namespace JinDouYun\Controller\Market;

use Mall\Framework\Core\ErrorCode;
use Mall\Framework\Core\StatusCode;

use JinDouYun\Controller\BaseController;
use JinDouYun\Model\Market\MCoupon;

class Coupon extends BaseController
{
    private $objMCoupon;

    /**
     * Coupon constructor.
     * @param bool $isCheckAcl
     * @param bool $isMustLogin
     * @throws \Exception
     */
    public function __construct($isCheckAcl = true, $isMustLogin = true)
    {
        parent::__construct($isCheckAcl, $isMustLogin);
        $this->objMCoupon = new MCoupon($this->onlineUserId, $this->onlineEnterpriseId);
    }

    /**
     * 添加,编辑优惠券公共数据
     * @return array
     */
    public function commonFieldFilter()
    {
        $params = $this->request->getRawJson();
        if (empty($params)) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }
        $data = [
            'name'           => isset($params['name']) ? $params['name'] : '',
            'couponType'     => isset($params['couponType']) ? $params['couponType'] : StatusCode::$couponType['commonly'],//优惠券类型(10普通 20会员卡优惠券)
            'reducePrice'    => isset($params['reducePrice']) ? $params['reducePrice'] : '',//优惠券面值
            'minPrice'       => isset($params['minPrice']) ? $params['minPrice'] : '',//最低消费金额 0.00不限金额
            'grantType'      => isset($params['grantType']) ? $params['grantType'] : '',//发放方式 10主动领取 20定向发放 30注册领取 40在线支付赠送
            'totalNum'       => isset($params['totalNum']) ? $params['totalNum'] : '',//总发放量 -1不限量
            'allowNum'       => isset($params['allowNum']) ? $params['allowNum'] : '',//每人限制领取 -1不限量
            'useShop'        => isset($params['useShop']) ? $params['useShop'] : '',//使用范围
            'applyRange'     => isset($params['applyRange']) ? $params['applyRange'] : '',//商品范围 10全部商品 20指定分类 30指定品牌
            'isMutex'        => isset($params['isMutex']) ? $params['isMutex'] : StatusCode::$standard,//是否互斥
        ];

        foreach ($data as $key => $value) {
            if ($value === '') {
                parent::sendOutput($key . '参数错误', ErrorCode::$paramError);
            }
        }

        $data['customerIds'] = isset($params['customerIds']) ? $params['customerIds'] : '';
        $data['startTime'] = isset($params['startTime']) ? $params['startTime'] : '';
        $data['endTime'] = isset($params['endTime']) ? $params['endTime'] : '';
        $data['grantStartTime'] = isset($params['grantStartTime']) ? $params['grantStartTime'] : '';//发放开始时间
        $data['grantEndTime'] = isset($params['grantEndTime']) ? $params['grantEndTime'] : '';//发放结束时间



        $data['customerSourceId'] = isset($params['customerSourceId']) ? $params['customerSourceId'] : 0;//领取范围

        if ($data['grantType'] == StatusCode::$grantType['onlinePay']) {
            if (!isset($params['mustCondition'])) {
                parent::sendOutput('mustCondition参数错误', ErrorCode::$paramError);
            }
            $data['mustCondition'] = $params['mustCondition'];//与在线值配合使用
        }
        if ($data['applyRange'] == StatusCode::$applyRange['appointCategory']) {
            if (!isset($params['categoryCollect'])) {
                parent::sendOutput('categoryCollect参数错误', ErrorCode::$paramError);
            }
            $data['categoryCollect'] = $params['categoryCollect'];
        }
        if ($data['applyRange'] == StatusCode::$applyRange['appointBrand']) {
            if (!isset($params['brandCollect'])) {
                parent::sendOutput('brandCollect参数错误', ErrorCode::$paramError);
            }
            $data['brandCollect'] = $params['brandCollect'];
        }
        if($data['applyRange'] == StatusCode::$applyRange['goodsCollect']){
            if (!isset($params['goodsCollect']) || empty($params['goodsCollect'])){
                parent::sendOutput('goodsCollect参数有误',ErrorCode::$paramError);
            }
            $data['goodsCollect'] = $params['goodsCollect'];
        }
        $data['remark'] = isset($params['remark']) ? $params['remark'] : '';//优惠券使用说明
        $data['createTime'] = time();
        $data['updateTime'] = time();
        return $data;
    }

    /**
     * 添加
     */
    public function add()
    {
        $data = $this->commonFieldFilter();
        $result = $this->objMCoupon->add($data);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 详情
     * @throws \Exception
     */
    public function getInfo()
    {
        $id = $this->request->param('request_id');
        if (empty($id)) {
            $this->sendOutput('参数错误', ErrorCode::$paramError);
        }
        $result = $this->objMCoupon->getInfo($id);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 列表
     */
    public function getAll()
    {
        $page = $this->request->param('page') ?: 1;
        $pageSize = $this->request->param('pageSize') ?: 10;
        $offset = ($page - 1) * $pageSize;
        $params['limit'] = $pageSize;
        $params['offset'] = $offset;

        $grantType = $this->request->param('grantType');
        $couponType = $this->request->param('couponType');
        $auditStatus = $this->request->param('auditStatus');
        if (!empty($grantType)) $params['grantType'] = $grantType;
        if (!empty($couponType)) $params['couponType'] = $couponType;
        if (!empty($auditStatus)) $params['auditStatus'] = $auditStatus;

        $orderData = $this->objMCoupon->getAll($params);
        if ($orderData->isSuccess()) {
            $returnData = $orderData->getData();
            $pageData = [
                'pageIndex' => $page,
                'pageSize'  => $pageSize,
                'pageTotal' => $returnData['total'],
            ];
            parent::sendOutput($returnData['data'], 0, $pageData);
        }
        parent::sendOutput($orderData->getData(), ErrorCode::$dberror);
    }

    /**
     * 更新审核状态
     */
    public function updateAuditStatus()
    {
        $params = $this->request->getRawJson();
        if (empty($params)) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }
        $data = [
            'id'          => isset($params['id']) ? $params['id'] : '',
            'auditStatus' => isset($params['auditStatus']) ? $params['auditStatus'] : '',
        ];
        foreach ($data as $k => $v) {
            if (empty($v) && $v !== 0) {
                $this->sendOutput($k . '参数错误', ErrorCode::$paramError);
            }
        }
        $result = $this->objMCoupon->updateAuditStatus($data);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 删除
     */
    public function del()
    {
        $id = $this->request->param('request_id');
        if (empty($id)) {
            $this->sendOutput('参数错误', ErrorCode::$paramError);
        }
        $result = $this->objMCoupon->del($id);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 编辑
     */
    public function edit()
    {
        $id = $this->request->param('request_id');
        if (empty($id)) {
            $this->sendOutput('参数错误', ErrorCode::$paramError);
        }
        $data = $this->commonFieldFilter();
        $result = $this->objMCoupon->edit($data, $id);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 搜索
     */
    public function search()
    {
        $params = $this->request->getRawJson();
        if (empty($params)) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }

        $selectParams = [
            'name'        => isset($params['name']) ? $params['name'] : '',//优惠券名称
            'grantType'   => isset($params['grantType']) ? $params['grantType'] : '',//发放方式
            'applyRange'  => isset($params['applyRange']) ? $params['applyRange'] : '',//适用范围
            'auditStatus' => isset($params['auditStatus']) ? $params['auditStatus'] : '',//审核状态
            'start'       => isset($params['start']) ? $params['start'] : '',
            'end'         => isset($params['end']) ? $params['end'] : '',
            'useShop'     => isset($params['useShop']) ? $params['useShop'] : '',
        ];

        $pageParams = pageToOffset(isset($params['page']) ? $params['page'] : 1, isset($params['pageSize']) ? $params['pageSize'] : 10);
        $selectParams['limit'] = $pageParams['limit'];
        $selectParams['offset'] = $pageParams['offset'];

        $result = $this->objMCoupon->search($selectParams);
        if ($result->isSuccess()) {
            $returnData = $result->getData();
            $pageData = [
                'pageIndex' => $params['page'],
                'pageSize'  => $params['pageSize'],
                'pageTotal' => $returnData['total'],
            ];
            parent::sendOutput($returnData['data'], 0, $pageData);
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 启用/禁用
     */
    public function updateEnableStatus()
    {
        $params = $this->request->getRawJson();
        if (empty($params)) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }
        $data = [
            'id'           => isset($params['id']) ? $params['id'] : '',
            'enableStatus' => isset($params['enableStatus']) ? $params['enableStatus'] : '',
        ];
        foreach ($data as $k => $v) {
            if (empty($v) && $v !== 0) {
                $this->sendOutput($k . '参数错误', ErrorCode::$paramError);
            }
        }
        $result = $this->objMCoupon->updateEnableStatus($data);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }
}