<?php
/**
 * 配送方式设置
 * Created by PhpStorm.
 * User: XiaoMing
 * Date: 2019/10/31
 * Time: 15:46
 */

namespace JinDouYun\Controller\System;

use Mall\Framework\Core\ErrorCode;
use JinDouYun\Controller\BaseController;

use JinDouYun\Model\System\MDeliverySetting;

class DeliverySetting extends BaseController
{
    private $objMDeliverySetting;

    /**
     * DeliverySetting constructor.
     * @param bool $isCheckAcl
     * @param bool $isMustLogin
     * @throws \Exception
     */
    public function __construct($isCheckAcl = true, $isMustLogin = true)
    {
        parent::__construct($isCheckAcl, $isMustLogin);
        $this->objMDeliverySetting = new MDeliverySetting($this->onlineUserId, $this->onlineEnterpriseId);
    }

    /**
     * 添加,编辑配送方式公共数据
     * @return array
     */
    public function commonFieldFilter()
    {
        $params = $this->request->getRawJson();
        if (empty($params)) {
            $this->sendOutput('参数为空', ErrorCode::$paramError);
        }
        $data = [
            'title'         => isset($params['title']) ? $params['title'] : '',
            'defaultStatus' => isset($params['defaultStatus']) ? $params['defaultStatus'] : '',
            'enableStatus'  => isset($params['enableStatus']) ? $params['enableStatus'] : '',
        ];

        foreach ($data as $key => $value) {
            if (empty($value) && $value !== 0) {
                $this->sendOutput($key . '参数错误', ErrorCode::$paramError);
            }
        }

        $data['shopId'] = $this->shopId;
        $data['enterpriseId'] = $this->onlineEnterpriseId;
        $data['setData'] = getArrayItem( $params,'setData','');
        return $data;
    }

    /**
     * Doc: (des="配置运费模版")
     * User: XMing
     * Date: 2020/9/7
     * Time: 10:49 上午
     */
    public function setData()
    {
        //$id = $this->request->param('request_id');
        $params = $this->request->getRawJson();
        if (empty($params)) {
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }
        if (!isset($params['shopId'])){
            $params['shopId'] = is_null($this->shopId) ? 0 : $this->shopId;
        }
        $result = $this->objMDeliverySetting->setData($params);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * Doc: (des="设置自提点")
     * User: XMing
     * Date: 2020/9/9
     * Time: 10:28 上午
     */
    public function setSelfData()
    {
        $params = $this->request->getRawJson();
        if (empty($params)) {
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }
        if (!isset($params['shopId'])){
            $params['shopId'] = is_null($this->shopId) ? 0 : $this->shopId;
        }
        $result = $this->objMDeliverySetting->setSelfData($params);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());

    }

    /**
     * Doc: (des="删除运费模版规则")
     * User: XMing
     * Date: 2020/9/7
     * Time: 4:56 下午
     */
    public function delDeliveryRule()
    {
        $id = $this->request->param('request_id');
        $result = $this->objMDeliverySetting->delDeliveryRule($id);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * Doc: (des="删除自提点")
     * User: XMing
     * Date: 2020/9/9
     * Time: 10:55 上午
     */
    public function delDeliverySelfRule()
    {
        $id = $this->request->param('request_id');
        $result = $this->objMDeliverySetting->delDeliverySelfRule($id);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 获取配送方式列表
     * @throws \Exception
     */
    public function getAllDelivery()
    {
        $page = $this->request->param('page') ?: 1;
        $pageSize = $this->request->param('pageSize') ?: 10;
        $offset = ($page - 1) * $pageSize;
        $selectParams = [
            'limit'  => $pageSize,
            'offset' => $offset,
        ];
        $selectParams['shopId'] = $this->shopId;
        $orderData = $this->objMDeliverySetting->getAllDelivery($selectParams);
        if ($orderData->isSuccess()) {
            $returnData = $orderData->getData();
            $pageData = [
                'pageIndex' => $page,
                'pageSize'  => $pageSize,
                'pageTotal' => $returnData['total'],
            ];
            parent::sendOutput($returnData['data'], 0, $pageData);
        }
        parent::sendOutput($orderData->getData(), ErrorCode::$dberror);
    }

    /**
     * 更新配送方式,启用/禁用
     * @throws \Exception
     */
    public function updateEnableStatus()
    {
        $params['id'] = $this->request->param('request_id');
        $params['enableStatus'] = $this->request->param('enableStatus');

        foreach ($params as $key => $value) {
            if (empty($value)) {
                $this->sendOutput($key . '参数错误', ErrorCode::$paramError);
            }
        }

        $result = $this->objMDeliverySetting->updateEnableStatus($params);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 更新配送方式的默认状态
     * @throws \Exception
     */
    public function updateDefaultStatus()
    {
        $params['id'] = $this->request->param('request_id');
        $params['defaultStatus'] = $this->request->param('defaultStatus');

        foreach ($params as $key => $value) {
            if (empty($value)) {
                $this->sendOutput($key . '参数错误', ErrorCode::$paramError);
            }
        }

        $params['shopId'] = is_null($this->shopId) ? 0 : $this->shopId;
        $result = $this->objMDeliverySetting->updateDefaultStatus($params);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * Doc: (des="获取配送方式详情")
     * User: XMing
     * Date: 2020/9/7
     * Time: 12:29 下午
     */
    public function getInfo()
    {
        $id = $this->request->param('request_id');
        $shopId = is_null($this->shopId) ? 0 : $this->shopId;
        $result = $this->objMDeliverySetting->getInfo($id,$shopId);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * 用于获取配送方式(设置->配送方式列表不要使用此接口)
     */
    public function allDelivery()
    {
        $dbResult = $this->objMDeliverySetting->allDelivery();
        if ($dbResult->isSuccess()) {
            $returnData = $dbResult->getData();
            parent::sendOutput($returnData['data'], 0);
        }
        parent::sendOutput($dbResult->getData(), ErrorCode::$dberror);
    }

    /**
     * Doc: (des="设置默认运费模版")
     * User: XMing
     * Date: 2020/9/10
     * Time: 2:07 下午
     */
    public function setDefaultRule()
    {
        $id = $this->request->param('request_id');
        $shopId = is_null($this->shopId) ? 0 : $this->shopId;
        $result = $this->objMDeliverySetting->setDefaultRule($id,$shopId);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * Doc: (des="获取运费模版详情")
     * User: XMing
     * Date: 2020/9/8
     * Time: 6:22 下午
     */
    public function getRuleInfo()
    {
        $id = $this->request->param('request_id');
        $result = $this->objMDeliverySetting->getRuleInfo($id);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * Doc: (des="获取自提点详情")
     * User: XMing
     * Date: 2020/9/9
     * Time: 10:48 上午
     */
    public function getSelfRuleInfo()
    {
        $id = $this->request->param('request_id');
        $result = $this->objMDeliverySetting->getSelfRuleInfo($id);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }

    /**
     * Doc: (des="获取运费模版")
     * User: XMing
     * Date: 2020/9/7
     * Time: 6:26 下午
     */
    public function getAllExpressRule()
    {
        $page = $this->request->param('page') ?: 1;
        $pageSize = $this->request->param('pageSize') ?: 10;
        $offset = ($page - 1) * $pageSize;
        $orderData = $this->objMDeliverySetting->getAllExpressRule();
        if ($orderData->isSuccess()) {
            $returnData = $orderData->getData();
            $pageData = [
                'pageIndex' => $page,
                'pageSize'  => $pageSize,
                'pageTotal' => $returnData['total'],
            ];
            parent::sendOutput($returnData['data'], 0, $pageData);
        }
        parent::sendOutput($orderData->getData(), ErrorCode::$dberror);
    }

    /**
     * Doc: (des="获取快递公司")
     * User: XMing
     * Date: 2020/9/8
     * Time: 6:53 下午
     */
    public function getAllExpress()
    {
        $page = $this->request->param('page') ?: 1;
        $pageSize = $this->request->param('pageSize') ?: 10;
        $offset = ($page - 1) * $pageSize;
        $orderData = $this->objMDeliverySetting->getAllExpress();
        if ($orderData->isSuccess()) {
            $returnData = $orderData->getData();
            $pageData = [
                'pageIndex' => $page,
                'pageSize'  => $pageSize,
                'pageTotal' => $returnData['total'],
            ];
            parent::sendOutput($returnData['data'], 0, $pageData);
        }
        parent::sendOutput($orderData->getData(), ErrorCode::$dberror);
    }

    /**
     * Doc: (des="后台-获取自提点")
     * User: XMing
     * Date: 2020/11/4
     * Time: 3:30 下午
     */
    public function getAllSelfExpressRule()
    {
        $result = $this->objMDeliverySetting->getAllSelfExpressRule();
        if ($result->isSuccess()) {
            $pageData = [
                'pageIndex' => 1,
                'pageSize'  => 999,
                'pageTotal' => 999,
            ];
            parent::sendOutput($result->getData(),0,$pageData);
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }
    
    /**
     * 物流提醒
     */
    public function updateLogisticsReminder()
    {
        $params = $this->request->getRawJson();
        if( empty($params) ){
            $this->sendOutput('参数为空', ErrorCode::$paramError );
        }
        $reminderData = [
            'id'            => getArrayItem( $params,'id'),
            'reminder'       => getArrayItem( $params,'reminder','')
        ];
        foreach ($reminderData as $key => $value) {
            if (empty($value)) {
                $this->sendOutput($key . '参数错误', ErrorCode::$paramError);
            }
        }
        $result = $this->objMDeliverySetting->updateLogisticsReminder($reminderData);
        if ($result->isSuccess()) {
            parent::sendOutput($result->getData());
        }
        parent::sendOutput($result->getData(), $result->getErrorCode());
    }
    

}